import pytest


def test_get_valves_historical_accumulations_error(iccpro):
    with pytest.raises(RuntimeError):
        iccpro.get_valves_historical_accumulations()


def test_get_meters_historical_accumulations_error(iccpro):
    with pytest.raises(RuntimeError):
        iccpro.get_meters_historical_accumulations()



from urllib.parse import urljoin

from .core import process_response


class VirtualMetersMixin:
    url = "iccpro/api/virtualmeters"

    @process_response
    def get_virtual_meters_general_info(self):
        url = urljoin(self.host, VirtualMetersMixin.url)
        return self.make_request("GET", url)



from urllib.parse import urljoin

from .core import process_response


class TokenMixin:
    url = "iccpro/token"
    headers = {
        "content-type": "application/x-www-form-urlencoded",
    }

    @process_response
    def acquire_token(self):
        url = urljoin(self.host, TokenMixin.url)
        data = {
            "grant_type": "password",
            "username": self.username,
            "password": self.password,
            "client_id": self.client_id,
            "client_secret": self.client_secret,
        }
        return self.make_request("POST", url, headers=TokenMixin.headers, data=data)

    @process_response
    def refresh_token(self):
        url = urljoin(self.host, TokenMixin.url)
        data = {
            "grant_type": "refresh_token",
            "client_id": self.client_id,
            "client_secret": self.client_secret,
            "refresh_token": self._refresh_token,
        }
        return self.make_request("POST", url, headers=TokenMixin.headers, data=data)



from os import getenv

import pytest
from dotenv import load_dotenv

from icc_pro import ICC_PRO

load_dotenv()


@pytest.fixture()
def iccpro():
    host = getenv("iccpro_host")
    username = getenv("iccpro_username")
    password = getenv("iccpro_password")
    client_id = getenv("iccpro_client_id")
    client_secret = getenv("iccpro_client_secret")

    if all([host, username, password, client_id, client_secret]):
        login = True
    else:
        login = False

    return ICC_PRO(host, username, password, client_id, client_secret, login=login)



from os import getenv
from urllib.parse import urljoin

import pytest


def test_mock_get_programs_general_info(mocker, iccpro):
    m = mocker.patch("requests.Session.request")

    iccpro.get_programs_general_info()

    m.assert_called_once_with(
        "GET",
        urljoin(getenv("iccpro_host"), "iccpro/api/programs"),
    )


@pytest.mark.skipif(not getenv("iccpro_host"), reason="Missing config")
def test_get_programs_general_info(iccpro):
    assert iccpro.get_programs_general_info()


@pytest.mark.skipif(not getenv("iccpro_host"), reason="Missing config")
def test_get_programs_detailed_info(iccpro):
    assert iccpro.get_programs_detailed_info()


def test_set_programs_status_error(iccpro):
    with pytest.raises(NotImplementedError):
        iccpro.set_programs_status()


def test_set_programs_preset_data_error(iccpro):
    with pytest.raises(NotImplementedError):
        iccpro.set_programs_preset_data()


def test_irrigate_now_program_group_error(iccpro):
    with pytest.raises(NotImplementedError):
        iccpro.irrigate_now_program_group()


def test_reset_program_alarm_error(iccpro):
    with pytest.raises(NotImplementedError):
        iccpro.reset_program_alarm()


def test_change_program_depth_error(iccpro):
    with pytest.raises(NotImplementedError):
        iccpro.change_program_depth()



from urllib.parse import urljoin

from requests.models import PreparedRequest

from .core import process_response, required_kwargs


class AccumulationsMixin:
    url = "iccpro/api/accumulation"

    @required_kwargs("fromdatetime", "todatetime", "resolution")
    @process_response
    def get_valves_historical_accumulations(self, **kwargs):
        url = urljoin(self.host, f"{AccumulationsMixin.url}/valves")
        req = PreparedRequest()
        req.prepare_url(url, kwargs)
        return self.make_request("GET", req.url)

    @required_kwargs("fromdatetime", "todatetime", "resolution")
    @process_response
    def get_meters_historical_accumulations(self, **kwargs):
        url = urljoin(self.host, f"{AccumulationsMixin.url}/meters")
        req = PreparedRequest()
        req.prepare_url(url, kwargs)
        return self.make_request("GET", req.url)



import requests
import wrapt


@wrapt.decorator
def process_response(wrapped, instance, args, kwargs):
    try:
        resp = wrapped(*args, **kwargs)
    except (requests.ConnectionError, requests.Timeout) as e:
        raise Exception("Service Unavailable") from e
    else:
        resp.raise_for_status()
        return resp.json()


def required_kwargs(*required_kwargs):
    """
    Decorator to check if criteria of required kwargs are met
    Raises:
        RuntimeError: a bad client request
    Returns:
        func: decorated function
    """

    @wrapt.decorator
    def wrapper(wrapped, instance, args, kwargs):
        if all([kwarg in kwargs for kwarg in required_kwargs]):
            return wrapped(*args, **kwargs)

        msg = f"required kwargs: {' and '.join(required_kwargs)}"

        raise RuntimeError(f"{msg}")

    return wrapper



from os import getenv

from icc_pro import ICC_PRO


def test_icc_pro_init(iccpro):
    assert len(ICC_PRO.list_generic_endpoints()) == 8

    host = getenv("iccpro_host")
    username = getenv("iccpro_username")
    password = getenv("iccpro_password")
    client_id = getenv("iccpro_client_id")
    client_secret = getenv("iccpro_client_secret")

    assert iccpro.host == host
    assert iccpro.username == username
    assert iccpro.password == password
    assert iccpro.client_id == client_id
    assert iccpro.client_secret == client_secret

    assert len(iccpro.list_generic_endpoints()) == 8
    assert len(iccpro.list_detailed_endpoints()) == 8



from urllib.parse import urljoin

from requests.models import PreparedRequest

from .core import process_response


class ProgramsMixin:
    url = "iccpro/api/programs"

    @process_response
    def get_programs_general_info(self):
        url = urljoin(self.host, ProgramsMixin.url)
        return self.make_request("GET", url)

    @process_response
    def get_programs_detailed_info(self, **kwargs):
        url = urljoin(self.host, f"{ProgramsMixin.url}/getprograms")
        req = PreparedRequest()
        req.prepare_url(url, kwargs)
        return self.make_request("GET", req.url)

    def set_programs_status(self):
        raise NotImplementedError

    def set_programs_preset_data(self):
        raise NotImplementedError

    def irrigate_now_program_group(self):
        raise NotImplementedError

    def reset_program_alarm(self):
        raise NotImplementedError

    def change_program_depth(self):
        raise NotImplementedError



from os import getenv

import pytest


@pytest.mark.skipif(not getenv("iccpro_host"), reason="Missing config")
def test_acquire_token(iccpro):
    assert iccpro.acquire_token()


@pytest.mark.skipif(not getenv("iccpro_host"), reason="Missing config")
def test_refresh_token(iccpro):
    assert iccpro.refresh_token()



from os import getenv
from urllib.parse import urljoin

import pytest


def test_mock_get_virtual_meters_general_info(mocker, iccpro):
    m = mocker.patch("requests.Session.request")

    iccpro.get_virtual_meters_general_info()

    m.assert_called_once_with(
        "GET",
        urljoin(getenv("iccpro_host"), "iccpro/api/virtualmeters"),
    )


@pytest.mark.skipif(not getenv("iccpro_host"), reason="Missing config")
def test_get_virtual_meters_general_info(iccpro):
    assert iccpro.get_virtual_meters_general_info()



from urllib.parse import urljoin

import requests

from .accumulations import AccumulationsMixin
from .analog_inputs import AnalogInputsMixin
from .field_units import FieldUnitsMixin
from .meters import MetersMixin
from .programs import ProgramsMixin
from .sensors import SensorsMixin
from .token import TokenMixin
from .valves import ValvesMixin
from .virtual_meters import VirtualMetersMixin

mixins = [
    AccumulationsMixin,
    FieldUnitsMixin,
    AnalogInputsMixin,
    MetersMixin,
    ProgramsMixin,
    SensorsMixin,
    TokenMixin,
    ValvesMixin,
    VirtualMetersMixin,
]


class ICC_PRO(*mixins):
    url = None

    def __init__(self, host, username, password, client_id, client_secret, login=True):
        self.host = host
        self.username = username
        self.password = password
        self.client_id = client_id
        self.client_secret = client_secret

        self.session = requests.Session()

        if login:
            self.set_tokens(self.acquire_token())

    def set_tokens(self, resp):
        self._access_token = resp.get("access_token")
        self._refresh_token = resp.get("refresh_token")
        self.session.headers.update(
            {
                "Authorization": f"Bearer {self._access_token}",
            }
        )

    def renew(self):
        self.set_tokens(self.refresh_token())

    @classmethod
    def list_generic_endpoints(cls):
        return [(m.__name__[:-5], m.url) for m in mixins]

    def list_detailed_endpoints(self):
        return [(m.__name__[:-5], urljoin(self.host, m.url)) for m in mixins]

    def make_request(self, method, url, **kwargs):
        return self.session.request(method, url, **kwargs)



from urllib.parse import urljoin

from requests.models import PreparedRequest

from .core import process_response


class ValvesMixin:
    url = "iccpro/api/valves"

    @process_response
    def get_valves_general_info(self):
        url = urljoin(self.host, ValvesMixin.url)
        return self.make_request("GET", url)

    @process_response
    def get_valves_gis_info(self, **kwargs):
        url = urljoin(self.host, f"{ValvesMixin.url}/getgisdata")
        req = PreparedRequest()
        req.prepare_url(url, kwargs)
        return self.make_request("GET", req.url)

    @process_response
    def get_valves_status(self, **kwargs):
        url = urljoin(self.host, f"{ValvesMixin.url}/getgisstatus")
        req = PreparedRequest()
        req.prepare_url(url, kwargs)
        return self.make_request("GET", req.url)

    def create_valves_snapshot(self):
        raise NotImplementedError

    def update_valves_snapshot(self):
        raise NotImplementedError



from urllib.parse import urljoin

from requests.models import PreparedRequest

from .core import process_response, required_kwargs


class AnalogInputsMixin:
    url = "iccpro/api/analoginputs"

    @process_response
    def get_analog_inputs_general_info(self):
        url = urljoin(self.host, AnalogInputsMixin.url)
        return self.make_request("GET", url)

    @process_response
    def get_analog_inputs_current_data(self, **kwargs):
        url = urljoin(self.host, f"{AnalogInputsMixin.url}/current")
        req = PreparedRequest()
        req.prepare_url(url, kwargs)
        return self.make_request("GET", req.url)

    @required_kwargs("fromdatetime", "todatetime", "resolution")
    @process_response
    def get_analog_inputs_historical_data(self, **kwargs):
        url = urljoin(self.host, f"{AnalogInputsMixin.url}/history")
        req = PreparedRequest()
        req.prepare_url(url, kwargs)
        return self.make_request("GET", req.url)
